@@ -2,9 +2,10 @@
 use strict;
 use lib 'lib';
 use CPAN::WWW::Testers;
+$| = 1;
 
 my $directory = 'www';
-mkdir $directory;
+mkdir $directory || die $!;
 
 my $t = CPAN::WWW::Testers->new();
 $t->directory($directory);
